@@ -12,6 +12,7 @@ from functools import wraps
 from .config import BaseConfig
 from ..utils import _raise, load_json, save_json, axes_check_and_normalize, axes_dict, move_image_axes
 from ..utils.six import Path, FileNotFoundError
+from ..utils.tf import keras_import, IS_KERAS_3_PLUS
 from ..data import Normalizer, NoNormalizer
 from ..data import Resizer, NoResizer
 from .pretrained import get_model_details, get_model_instance, get_registered_models
@@ -35,6 +36,56 @@ def suppress_without_basedir(warn):
 
 
 
+if IS_KERAS_3_PLUS:
+    import h5py
+    from packaging.version import Version
+    from keras.src.legacy.saving import legacy_h5_format
+    from keras.src import backend, __version__ as keras_version
+
+    if Version(keras_version) >= Version("3.3.0"):
+        save_weights_to_hdf5_group = legacy_h5_format.save_weights_to_hdf5_group
+    else:
+        def save_weights_to_hdf5_group(f, model):
+            legacy_h5_format.save_attributes_to_hdf5_group(f, "layer_names", [layer.name.encode("utf8") for layer in model.layers])
+            f.attrs["backend"] = backend.backend().encode("utf8")
+            f.attrs["keras_version"] = str(keras_version).encode("utf8")
+            for layer in sorted(model.layers, key=lambda x: x.name):
+                g = f.create_group(layer.name)
+                weights = legacy_h5_format._legacy_weights(layer)
+                save_subset_weights_to_hdf5_group(g, weights)
+            g = f.create_group("top_level_model_weights")
+            weights = [v for v in model._trainable_variables + model._non_trainable_variables if v in model.weights]
+            save_subset_weights_to_hdf5_group(g, weights)
+
+        def save_subset_weights_to_hdf5_group(f, weights):
+            # FIX: use w.path instead of w.name to avoid name collisions (for "functional" layers)
+            # -> has been fixed since keras 3.3.0: https://github.com/keras-team/keras/blob/v3.3.0/keras/src/legacy/saving/legacy_h5_format.py#L234
+            weight_names = [w.path.encode("utf8") for w in weights]
+            weight_values = [backend.convert_to_numpy(w) for w in weights]
+            legacy_h5_format.save_attributes_to_hdf5_group(f, "weight_names", weight_names)
+            for name, val in zip(weight_names, weight_values):
+                param_dset = f.create_dataset(name, val.shape, dtype=val.dtype)
+                param_dset[() if not val.shape else slice(None)] = val
+
+    def _keras3_monkey_patch_legacy_weights(model):
+        ref_save_weights = model.save_weights
+
+        def save_weights(self, filepath, overwrite=True):
+            p = Path(filepath)
+            if not overwrite and p.exists():
+                raise FileExistsError(f"Weights file already exists: {str(p.resolve())}")
+            if p.name.endswith(".weights.h5"):
+                warnings.warn("Detected filename suffix '.weights.h5', thus saving in newer Keras 3.x file format (cannot be loaded with Keras 2.x)")
+            if not p.name.endswith(".weights.h5"):
+                with h5py.File(str(p), "w") as f:
+                    save_weights_to_hdf5_group(f, self)
+            else:
+                return ref_save_weights(filepath, overwrite=overwrite)
+
+        model.save_weights = save_weights.__get__(model)
+
+
+
 @add_metaclass(ABCMeta)
 class BaseModel(object):
     """Base model.
@@ -77,7 +128,7 @@ class BaseModel(object):
         try:
             get_model_details(cls, name_or_alias, verbose=True)
             return get_model_instance(cls, name_or_alias)
-        except ValueError:
+        except ValueError as e:
             if name_or_alias is not None:
                 print("Could not find model with name or alias '%s'" % (name_or_alias), file=sys.stderr)
                 sys.stderr.flush()
@@ -109,6 +160,9 @@ class BaseModel(object):
             self._update_and_check_config()
         self._model_prepared = False
         self.keras_model = self._build()
+        if IS_KERAS_3_PLUS and isinstance(self.keras_model, keras_import('models', 'Model')):
+            # monkey-patch keras model to save weights in legacy format if suffix is not '.weights.h5'
+            _keras3_monkey_patch_legacy_weights(self.keras_model)
         if config is None:
             self._find_and_load_weights()
 
@@ -137,6 +191,7 @@ class BaseModel(object):
         if self.config is None:
             if config_file.exists():
                 config_dict = load_json(str(config_file))
+                config_dict = self._config_class.update_loaded_config(config_dict)
                 self.config = self._config_class(**config_dict)
                 if not self.config.is_valid():
                     invalid_attr = self.config.is_valid(True)[1]
@@ -189,10 +244,16 @@ class BaseModel(object):
         if self.basedir is not None:
             from ..utils.tf import keras_import
             ModelCheckpoint = keras_import('callbacks', 'ModelCheckpoint')
+            # keras 3: need to add suffix to filename because ModelCheckpoint constructor throws error if it's missing
+            suffix = ".weights.h5" if IS_KERAS_3_PLUS else ""
             if self.config.train_checkpoint is not None:
-                callbacks.append(ModelCheckpoint(str(self.logdir / self.config.train_checkpoint),       save_best_only=True,  save_weights_only=True))
+                callbacks.append(ModelCheckpoint(str(self.logdir / self.config.train_checkpoint) + suffix,  save_best_only=True,  save_weights_only=True))
+                # keras3: remove suffix because patched model.save_weights can save in legacy format
+                if IS_KERAS_3_PLUS: callbacks[-1].filepath = callbacks[-1].filepath[:-len(suffix)]
             if self.config.train_checkpoint_epoch is not None:
-                callbacks.append(ModelCheckpoint(str(self.logdir / self.config.train_checkpoint_epoch), save_best_only=False, save_weights_only=True))
+                callbacks.append(ModelCheckpoint(str(self.logdir / self.config.train_checkpoint_epoch) + suffix, save_best_only=False, save_weights_only=True))
+                # keras3: remove suffix because patched model.save_weights can save in legacy format
+                if IS_KERAS_3_PLUS: callbacks[-1].filepath = callbacks[-1].filepath[:-len(suffix)]
         return callbacks
 
 
