@@ -2,9 +2,7 @@
 from __future__ import print_function, unicode_literals, absolute_import, division
 
 import warnings
-
 import numpy as np
-import tensorflow as tf
 from six import string_types
 
 from csbdeep.internals.probability import ProbabilisticPrediction
@@ -13,13 +11,23 @@ from .base_model import BaseModel, suppress_without_basedir
 
 from ..utils import _raise, axes_check_and_normalize, axes_dict, move_image_axes
 from ..utils.six import Path
-from ..utils.tf import export_SavedModel
+from ..utils.tf import export_SavedModel, IS_TF_1, keras_import, CARETensorBoardImage
 from ..version import __version__ as package_version
 from ..data import Normalizer, NoNormalizer, PercentileNormalizer
 from ..data import Resizer, NoResizer, PadAndCropResizer
 from ..internals.predict import predict_tiled, tile_overlap, Progress, total_n_tiles
 from ..internals import nets, train
 
+from packaging.version import Version
+keras = keras_import()
+
+import tensorflow as tf
+# if IS_TF_1:
+#     import tensorflow as tf
+# else:
+#     import tensorflow.compat.v1 as tf
+#     # tf.disable_v2_behavior()
+
 
 class CARE(BaseModel):
     """Standard CARE network for image restoration and enhancement.
@@ -100,28 +108,34 @@ class CARE(BaseModel):
 
         """
         if optimizer is None:
-            from keras.optimizers import Adam
-            optimizer = Adam(lr=self.config.train_learning_rate)
+            Adam = keras_import('optimizers', 'Adam')
+            learning_rate = 'lr' if Version(getattr(keras, '__version__', '9.9.9')) < Version('2.3.0') else 'learning_rate'
+            optimizer = Adam(**{learning_rate: self.config.train_learning_rate})
         self.callbacks = train.prepare_model(self.keras_model, optimizer, self.config.train_loss, **kwargs)
 
         if self.basedir is not None:
             self.callbacks += self._checkpoint_callbacks()
 
             if self.config.train_tensorboard:
-                from ..utils.tf import CARETensorBoard
-                self.callbacks.append(CARETensorBoard(log_dir=str(self.logdir), prefix_with_timestamp=False, n_images=3, write_images=True, prob_out=self.config.probabilistic))
+                if IS_TF_1:
+                    from ..utils.tf import CARETensorBoard
+                    self.callbacks.append(CARETensorBoard(log_dir=str(self.logdir), prefix_with_timestamp=False, n_images=3, write_images=True, prob_out=self.config.probabilistic))
+                else:
+                    from tensorflow.keras.callbacks import TensorBoard
+                    self.callbacks.append(TensorBoard(log_dir=str(self.logdir/'logs'), write_graph=False, profile_batch=0))
 
         if self.config.train_reduce_lr is not None:
-            from keras.callbacks import ReduceLROnPlateau
+            ReduceLROnPlateau = keras_import('callbacks', 'ReduceLROnPlateau')
             rlrop_params = self.config.train_reduce_lr
             if 'verbose' not in rlrop_params:
                 rlrop_params['verbose'] = True
-            self.callbacks.append(ReduceLROnPlateau(**rlrop_params))
+            # TF2: add as first callback to put 'lr' in the logs for TensorBoard
+            self.callbacks.insert(0,ReduceLROnPlateau(**rlrop_params))
 
         self._model_prepared = True
 
 
-    def train(self, X,Y, validation_data, epochs=None, steps_per_epoch=None):
+    def train(self, X,Y, validation_data, epochs=None, steps_per_epoch=None, augmenter=None):
         """Train the neural network with the given data.
 
         Parameters
@@ -170,11 +184,18 @@ class CARE(BaseModel):
         if not self._model_prepared:
             self.prepare_for_training()
 
-        training_data = train.DataWrapper(X, Y, self.config.train_batch_size)
+        if (self.config.train_tensorboard and self.basedir is not None and
+            not IS_TF_1 and not any(isinstance(cb,CARETensorBoardImage) for cb in self.callbacks)):
+            self.callbacks.append(CARETensorBoardImage(model=self.keras_model, data=validation_data,
+                                                       log_dir=str(self.logdir/'logs'/'images'),
+                                                       n_images=3, prob_out=self.config.probabilistic))
 
-        history = self.keras_model.fit_generator(generator=training_data, validation_data=validation_data,
-                                                 epochs=epochs, steps_per_epoch=steps_per_epoch,
-                                                 callbacks=self.callbacks, verbose=1)
+        training_data = train.DataWrapper(X, Y, self.config.train_batch_size, length=epochs*steps_per_epoch, augmenter=augmenter)
+
+        fit = self.keras_model.fit_generator if IS_TF_1 else self.keras_model.fit
+        history = fit(iter(training_data), validation_data=validation_data,
+                      epochs=epochs, steps_per_epoch=steps_per_epoch,
+                      callbacks=self.callbacks, verbose=1)
         self._training_finished()
 
         return history
@@ -312,7 +333,7 @@ class CARE(BaseModel):
         # _permute_axes_n_tiles: (img_axes_in <-> net_axes_in) to convert n_tiles between img and net axes
         def _permute_n_tiles(n,undo=False):
             # hack: move tiling axis around in the same way as the image was permuted by creating an array
-            return _permute_axes_n_tiles(np.empty(n,np.bool),undo=undo).shape
+            return _permute_axes_n_tiles(np.empty(n,bool),undo=undo).shape
 
         # to support old api: set scalar n_tiles value for the largest tiling axis
         if np.isscalar(n_tiles) and int(n_tiles)==n_tiles and 1<=n_tiles:
