@@ -8,9 +8,8 @@ import warnings
 
 from packaging.version import Version
 
-from ..utils.tf import keras_import
+from ..utils.tf import keras_import, BACKEND as K
 keras = keras_import()
-K = keras_import('backend')
 
 from ..utils import _raise, axes_check_and_normalize, axes_dict, backend_channels_last
 
@@ -183,7 +182,8 @@ class Config(BaseConfig):
         self.train_tensorboard     = True
 
         # the parameter 'min_delta' was called 'epsilon' for keras<=2.1.5
-        min_delta_key = 'epsilon' if Version(keras.__version__)<=Version('2.1.5') else 'min_delta'
+        # keras.__version__ was removed in tensorflow 2.13.0
+        min_delta_key = 'epsilon' if Version(getattr(keras, '__version__', '9.9.9'))<=Version('2.1.5') else 'min_delta'
         self.train_reduce_lr       = {'factor': 0.5, 'patience': 10, min_delta_key: 0}
 
         # disallow setting 'n_dim' manually
